@@ -79,9 +79,9 @@ end
 -- @return iterator over keys
 function seq.keys(t)
   assert_arg(1,t,'table')
-  local key,value
+  local key
   return function()
-    key,value = next(t,key)
+    key = next(t,key)
     return key
   end
 end
@@ -434,15 +434,12 @@ end
 -- @param iter a sequence
 function seq.last (iter)
     iter = default_iter(iter)
-    local l = iter()
-    if l == nil then return nil end
+    local val, l = iter(), nil
+    if val == nil then return list{} end
     return function ()
-        local val,ll
-        val = iter()
+        val,l = iter(),val
         if val == nil then return nil end
-        ll = l
-        l = val
-        return val,ll
+        return val,l
     end
 end
 
@@ -514,7 +511,6 @@ setmetatable(seq,{
 -- @param ... for Lua 5.2 only, optional format specifiers, as in `io.read`.
 -- @return a sequence wrapper
 function seq.lines (f,...)
-    local n = select('#',...)
     local iter,obj
     if f == 'STDIN' then
         f = io.stdin
